


# This script builds a parameter file for geneid given a set of
# annotations and a fasta file

# local must be changed to the variable


FASTA_FILE=$2
GFF_FILE=$1
CEGMATMP="/tmp";

$CEGMA/bin/gff2zff $GFF_FILE \
  > $CEGMATMP/geneid.$$.selected.ann 

mkdir geneid_params

cd geneid_params

fathom $CEGMATMP/geneid.$$.selected.ann ../$FASTA_FILE \
   -export 100                           

fathom export.ann export.dna -extract Intron \
  | awk '{if ($1 ~ ">") c=0; else c++; if (c<100) print}' > intron.fa

fathom  export.ann export.dna -extract Donor -length 60 -offset 30 \
  > donor.fa

fathom  export.ann export.dna -extract Acceptor -length 60 -offset 30 \
  > acceptor.fa

fathom  export.ann export.dna -extract Start -length 60 -offset 30 \
  > start.fa

fathom  export.ann export.dna -extract Stop  -length 60 -offset 30 \
  > stop.fa


# Generating the initial and transition matrices
# with out stop codon

$CEGMA/bin/FastaToTbl export.tx | awk '{print $1,substr($2,1,length($2)-3)}' \
   |  $CEGMA/bin/MarkovMatrices 5 cds
sort +1 -2  -o cds.5.initial cds.5.initial
sort +1 -2  -o cds.5.transition cds.5.transition

# Generating the initial and transition matrices for noncoding
head -10000 intron.fa | $CEGMA/bin/FastaToTbl \
  | $CEGMA/bin/MarkovMatrices-noframe 5 intron
sort -o intron.5.initial intron.5.initial
sort -o intron.5.transition intron.5.transition

#  Compute log-likelihood exon matrices, assuming intron matrices describing background probabilities.
$CEGMA/bin/pro2log_ini intron.5.initial cds.5.initial \
  >  cds-intron.5.initial
$CEGMA/bin/pro2log_tran  intron.5.transition  cds.5.transition \
  >  cds-intron.5.transition

# Convert to geneid format
awk 'BEGIN {p=-1}{if (((NR+2) % 3)==0) p+=1; print $2,p,$1,$3}' \
  cds-intron.5.initial > cds-intron.5.initial.geneid
awk 'BEGIN {p=-1}{if (((NR+2) % 3)==0) p+=1; print $2,p,$1,$4}' \
  cds-intron.5.transition > cds-intron.5.transition.geneid

## Donors
# Obtaining false donor sites
$CEGMA/bin/FastaToTbl intron.fa export.tx > seq.tbl

$CEGMA/bin/FindRegexp GT < seq.tbl \
 | join  seq.tbl  - | $CEGMA/bin/BoundarySites2 2 30 \
 | sort | grep -v nn\
 > GTs
# Obtaining real donor sites
$CEGMA/bin/FastaToTbl donor.fa > donor.tbl

$CEGMA/bin/frequency 1 GTs \
  | head -240 > freq.GTs.1
$CEGMA/bin/frequency 1 donor.tbl \
  > freq.donor.1

awk -f $CEGMA/bin/aux3 freq.GTs.1 \
  freq.donor.1 > log_mat.donors
awk -f $CEGMA/bin/files 28 40 log_mat.donors \
  > donor.geneid                                                                            
## Acceptors
# Obtaining false acceptor sites

$CEGMA/bin/FindRegexp AG < seq.tbl \
 | join  seq.tbl  - | $CEGMA/bin/BoundarySites2 2 29 \
 | sort | grep -v nn\
 > AGs
# Obtaining real donor sites
$CEGMA/bin/FastaToTbl acceptor.fa > acceptor.tbl

$CEGMA/bin/frequency 1 AGs \
  | head -240 > freq.AGs.1
$CEGMA/bin/frequency 1 acceptor.tbl \
  | grep [ACGT] > freq.acceptor.1
awk -f $CEGMA/bin/aux3 freq.AGs.1 \
  freq.acceptor.1 > log_mat.acceptor
awk -f $CEGMA/bin/files 12 33 log_mat.acceptor \
  > acceptor.geneid

## Start
# Obtaining false start sites
$CEGMA/bin/FastaToTbl start.fa > start.tbl

$CEGMA/bin/FindRegexp ATG < seq.tbl \
 | join  seq.tbl  - | $CEGMA/bin/BoundarySites2 2 30 \
 | sort | grep -v nn\
 > ATGs

# Obtaining real donor sites
$CEGMA/bin/frequency 1 ATGs \
  | head -240 > freq.ATGs.1
$CEGMA/bin/frequency 1 start.tbl \
  > freq.start.1
awk -f $CEGMA/bin/aux3 freq.ATGs.1 \
  freq.start.1 > log_mat.start
awk -f $CEGMA/bin/files 27 35 log_mat.start \
  > start.geneid                                           

# Now we need just to put all the files in the geneid parameter file


$CEGMA/bin/make_paramfile $CEGMA/data/self.param.template \
     cds-intron.5.initial.geneid cds-intron.5.transition.geneid \
     start.geneid acceptor.geneid donor.geneid \
  > self.param